@@ -28,8 +28,6 @@
 #include <stdio.h>
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "winerror.h"
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -442,7 +440,7 @@ DWORD WINAPI GetLongPathNameA( LPCSTR shortpath, LPSTR longpath, DWORD longlen )
  */
 DWORD WINAPI GetShortPathNameW( LPCWSTR longpath, LPWSTR shortpath, DWORD shortlen )
 {
-    WCHAR               tmpshortpath[MAX_PATHNAME_LEN];
+    WCHAR               *tmpshortpath;
     LPCWSTR             p;
     DWORD               sp = 0, lp = 0;
     DWORD               tmplen;
@@ -462,12 +460,28 @@ DWORD WINAPI GetShortPathNameW( LPCWSTR longpath, LPWSTR shortpath, DWORD shortl
         return 0;
     }
 
-    /* check for drive letter */
-    if (longpath[0] != '/' && longpath[1] == ':' )
+    /* code below only removes characters from string, never adds, so this is
+     * the largest buffer that tmpshortpath will need to have */
+    tmpshortpath = HeapAlloc(GetProcessHeap(), 0, (strlenW(longpath) + 1) * sizeof(WCHAR));
+    if (!tmpshortpath)
     {
-        tmpshortpath[0] = longpath[0];
-        tmpshortpath[1] = ':';
-        sp = lp = 2;
+        SetLastError(ERROR_OUTOFMEMORY);
+        return 0;
+    }
+
+    if (longpath[0] == '\\' && longpath[1] == '\\' && longpath[2] == '?' && longpath[3] == '\\')
+    {
+        memcpy(tmpshortpath, longpath, 4 * sizeof(WCHAR));
+        sp = lp = 4;
+    }
+
+    /* check for drive letter */
+    if (longpath[lp] != '/' && longpath[lp + 1] == ':' )
+    {
+        tmpshortpath[sp] = longpath[lp];
+        tmpshortpath[sp + 1] = ':';
+        sp += 2;
+        lp += 2;
     }
 
     while (longpath[lp])
@@ -524,9 +538,11 @@ DWORD WINAPI GetShortPathNameW( LPCWSTR longpath, LPWSTR shortpath, DWORD shortl
         tmplen--; /* length without 0 */
     }
 
+    HeapFree(GetProcessHeap(), 0, tmpshortpath);
     return tmplen;
 
  notfound:
+    HeapFree(GetProcessHeap(), 0, tmpshortpath);
     TRACE("not found!\n" );
     SetLastError ( ERROR_FILE_NOT_FOUND );
     return 0;
@@ -621,13 +637,18 @@ DWORD WINAPI GetTempPathW( DWORD count, LPWSTR path )
 
     ret++; /* add space for terminating 0 */
 
-    if (count)
+    if (count >= ret)
     {
         lstrcpynW(path, tmp_path, count);
-        if (count >= ret)
-            ret--; /* return length without 0 */
-        else if (count < 4)
-            path[0] = 0; /* avoid returning ambiguous "X:" */
+        /* the remaining buffer must be zeroed up to 32766 bytes in XP or 32767
+         * bytes after it, we will assume the > XP behavior for now */
+        memset(path + ret, 0, (min(count, 32767) - ret) * sizeof(WCHAR));
+        ret--; /* return length without 0 */
+    }
+    else if (count)
+    {
+        /* the buffer must be cleared if contents will not fit */
+        memset(path, 0, count * sizeof(WCHAR));
     }
 
     TRACE("returning %u, %s\n", ret, debugstr_w(path));
@@ -929,7 +950,6 @@ DWORD WINAPI SearchPathW( LPCWSTR path, LPCWSTR name, LPCWSTR ext, DWORD buflen,
             if (!search)
             {
                 SetLastError( ERROR_OUTOFMEMORY );
-                HeapFree( GetProcessHeap(), 0, dll_path );
                 return 0;
             }
             strcpyW( search, name );
@@ -1954,3 +1974,90 @@ WCHAR * CDECL wine_get_dos_file_name( LPCSTR str )
         nt_name.Buffer[1] = '\\';
     return nt_name.Buffer;
 }
+
+/*************************************************************************
+ *           CreateSymbolicLinkW   (KERNEL32.@)
+ */
+BOOL WINAPI CreateSymbolicLinkW(LPCWSTR link, LPCWSTR target, DWORD flags)
+{
+    FIXME("(%s %s %d): stub\n", debugstr_w(link), debugstr_w(target), flags);
+    return TRUE;
+}
+
+/*************************************************************************
+ *           CreateSymbolicLinkA   (KERNEL32.@)
+ */
+BOOL WINAPI CreateSymbolicLinkA(LPCSTR link, LPCSTR target, DWORD flags)
+{
+    FIXME("(%s %s %d): stub\n", debugstr_a(link), debugstr_a(target), flags);
+    return TRUE;
+}
+
+/*************************************************************************
+ *           CreateHardLinkTransactedA   (KERNEL32.@)
+ */
+BOOL WINAPI CreateHardLinkTransactedA(LPCSTR link, LPCSTR target, LPSECURITY_ATTRIBUTES sa, HANDLE transaction)
+{
+    FIXME("(%s %s %p %p): stub\n", debugstr_a(link), debugstr_a(target), sa, transaction);
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return FALSE;
+}
+
+/*************************************************************************
+ *           CreateHardLinkTransactedW   (KERNEL32.@)
+ */
+BOOL WINAPI CreateHardLinkTransactedW(LPCWSTR link, LPCWSTR target, LPSECURITY_ATTRIBUTES sa, HANDLE transaction)
+{
+    FIXME("(%s %s %p %p): stub\n", debugstr_w(link), debugstr_w(target), sa, transaction);
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return FALSE;
+}
+
+/*************************************************************************
+ *           CheckNameLegalDOS8Dot3A   (KERNEL32.@)
+ */
+BOOL WINAPI CheckNameLegalDOS8Dot3A(const char *name, char *oemname, DWORD oemname_len,
+        BOOL *contains_spaces, BOOL *is_legal)
+{
+    WCHAR *nameW;
+
+    TRACE("(%s %p %u %p %p)\n", name, oemname,
+            oemname_len, contains_spaces, is_legal);
+
+    if (!name || !is_legal)
+        return FALSE;
+
+    if (!(nameW = FILE_name_AtoW( name, FALSE ))) return FALSE;
+
+    return CheckNameLegalDOS8Dot3W( nameW, oemname, oemname_len, contains_spaces, is_legal );
+}
+
+/*************************************************************************
+ *           CheckNameLegalDOS8Dot3W   (KERNEL32.@)
+ */
+BOOL WINAPI CheckNameLegalDOS8Dot3W(const WCHAR *name, char *oemname, DWORD oemname_len,
+        BOOL *contains_spaces_ret, BOOL *is_legal)
+{
+    OEM_STRING oem_str;
+    UNICODE_STRING nameW;
+    BOOLEAN contains_spaces;
+
+    TRACE("(%s %p %u %p %p)\n", wine_dbgstr_w(name), oemname,
+          oemname_len, contains_spaces_ret, is_legal);
+
+    if (!name || !is_legal)
+        return FALSE;
+
+    RtlInitUnicodeString( &nameW, name );
+
+    if (oemname) {
+        oem_str.Length = oemname_len;
+        oem_str.MaximumLength = oemname_len;
+        oem_str.Buffer = oemname;
+    }
+
+    *is_legal = RtlIsNameLegalDOS8Dot3( &nameW, oemname ? &oem_str : NULL, &contains_spaces );
+    if (contains_spaces_ret) *contains_spaces_ret = contains_spaces;
+
+    return TRUE;
+}
