@@ -39,10 +39,9 @@
 #include <machine/limits.h>
 #endif
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
+#define NONAMELESSUNION
 #include "windef.h"
 #include "winbase.h"
 #include "winternl.h"
@@ -1098,11 +1097,29 @@ BOOL WINAPI GetSystemTimes(LPFILETIME lpIdleTime, LPFILETIME lpKernelTime, LPFIL
 /***********************************************************************
  *           GetDynamicTimeZoneInformation   (KERNEL32.@)
  */
-DWORD WINAPI GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION info)
+DWORD WINAPI GetDynamicTimeZoneInformation(DYNAMIC_TIME_ZONE_INFORMATION *tzinfo)
 {
-    FIXME("(%p) stub!\n", info);
+    NTSTATUS status;
+
+    status = RtlQueryDynamicTimeZoneInformation( (RTL_DYNAMIC_TIME_ZONE_INFORMATION*)tzinfo );
+    if ( status != STATUS_SUCCESS )
+    {
+        SetLastError( RtlNtStatusToDosError(status) );
+        return TIME_ZONE_ID_INVALID;
+    }
+    return TIME_ZoneID( (TIME_ZONE_INFORMATION*)tzinfo );
+}
+
+/***********************************************************************
+ *           QueryThreadCycleTime   (KERNEL32.@)
+ */
+BOOL WINAPI QueryThreadCycleTime(HANDLE thread, PULONG64 cycle)
+{
+    static int once;
+    if (!once++)
+        FIXME("(%p,%p,): stub!\n", thread, cycle);
     SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return TIME_ZONE_ID_INVALID;
+    return FALSE;
 }
 
 /***********************************************************************
