@@ -81,6 +81,9 @@ static const WCHAR onerrorW[] = {'o','n','e','r','r','o','r',0};
 static const WCHAR focusW[] = {'f','o','c','u','s',0};
 static const WCHAR onfocusW[] = {'o','n','f','o','c','u','s',0};
 
+static const WCHAR focusinW[] = {'f','o','c','u','s','i','n',0};
+static const WCHAR onfocusinW[] = {'o','n','f','o','c','u','s','i','n',0};
+
 static const WCHAR helpW[] = {'h','e','l','p',0};
 static const WCHAR onhelpW[] = {'o','n','h','e','l','p',0};
 
@@ -190,6 +193,8 @@ static const event_info_t event_info[] = {
         EVENT_BIND_TO_BODY},
     {focusW,             onfocusW,             EVENTT_HTML,   DISPID_EVMETH_ONFOCUS,
         EVENT_DEFAULTLISTENER},
+    {focusinW,           onfocusinW,           EVENTT_HTML,   DISPID_EVMETH_ONFOCUSIN,
+        EVENT_BUBBLE},
     {helpW,              onhelpW,              EVENTT_KEY,    DISPID_EVMETH_ONHELP,
         EVENT_BUBBLE|EVENT_CANCELABLE},
     {keydownW,           onkeydownW,           EVENTT_KEY,    DISPID_EVMETH_ONKEYDOWN,
@@ -201,7 +206,7 @@ static const event_info_t event_info[] = {
     {loadW,              onloadW,              EVENTT_HTML,   DISPID_EVMETH_ONLOAD,
         EVENT_BIND_TO_BODY},
     {mousedownW,         onmousedownW,         EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEDOWN,
-        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE},
     {mousemoveW,         onmousemoveW,         EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEMOVE,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {mouseoutW,          onmouseoutW,          EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEOUT,
@@ -260,6 +265,7 @@ typedef struct {
     HTMLDOMNode *target;
     const event_info_t *type;
     nsIDOMEvent *nsevent;
+    VARIANT return_value;
     BOOL prevent_default;
     BOOL cancel_bubble;
 } HTMLEventObj;
@@ -464,6 +470,7 @@ static HRESULT WINAPI HTMLEventObj_put_returnValue(IHTMLEventObj *iface, VARIANT
         return DISP_E_BADVARTYPE;
     }
 
+    This->return_value = v;
     if(!V_BOOL(&v))
         This->prevent_default = TRUE;
     return S_OK;
@@ -473,10 +480,10 @@ static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
-    FIXME("(%p)->(%p)\n", This, p);
+    TRACE("(%p)->(%p)\n", This, p);
 
     V_VT(p) = VT_EMPTY;
-    return S_OK;
+    return VariantCopy(p, &This->return_value);
 }
 
 static HRESULT WINAPI HTMLEventObj_put_cancelBubble(IHTMLEventObj *iface, VARIANT_BOOL v)
@@ -551,7 +558,7 @@ static HRESULT WINAPI HTMLEventObj_get_keyCode(IHTMLEventObj *iface, LONG *p)
 static HRESULT WINAPI HTMLEventObj_get_button(IHTMLEventObj *iface, LONG *p)
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
-    UINT16 button = 0;
+    INT16 button = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -877,6 +884,19 @@ HRESULT create_event_obj(IHTMLEventObj **ret)
     return S_OK;
 }
 
+static inline event_target_t *get_event_target_data(EventTarget *event_target, BOOL alloc)
+{
+    event_target_t **ptr;
+
+    ptr = event_target->dispex.data->vtbl && event_target->dispex.data->vtbl->get_event_target_ptr
+        ? event_target->dispex.data->vtbl->get_event_target_ptr(&event_target->dispex)
+        : &event_target->ptr;
+    if(*ptr || !alloc)
+        return *ptr;
+
+    return *ptr = heap_alloc_zero(sizeof(event_target_t));
+}
+
 static HRESULT call_disp_func(IDispatch *disp, DISPPARAMS *dp, VARIANT *retv)
 {
     IDispatchEx *dispex;
@@ -1101,7 +1121,7 @@ static void fire_event_obj(HTMLDocumentNode *doc, eventid_t eid, HTMLEventObj *e
         do {
             hres = get_node(doc, nsnode, FALSE, &node);
             if(SUCCEEDED(hres) && node) {
-                call_event_handlers(doc, event_obj, *get_node_event_target(node),
+                call_event_handlers(doc, event_obj, get_event_target_data(&node->event_target, FALSE),
                         node->cp_container, eid, script_this ? script_this : (IDispatch*)&node->IHTMLDOMNode_iface);
                 node_release(node);
             }
@@ -1130,7 +1150,7 @@ static void fire_event_obj(HTMLDocumentNode *doc, eventid_t eid, HTMLEventObj *e
             if(NS_SUCCEEDED(nsres) && nsbody) {
                 hres = get_node(doc, (nsIDOMNode*)nsbody, FALSE, &node);
                 if(SUCCEEDED(hres) && node) {
-                    call_event_handlers(doc, event_obj, *get_node_event_target(node),
+                    call_event_handlers(doc, event_obj, get_event_target_data(&node->event_target, FALSE),
                             node->cp_container, eid, script_this ? script_this : (IDispatch*)&node->IHTMLDOMNode_iface);
                     node_release(node);
                 }
@@ -1140,7 +1160,7 @@ static void fire_event_obj(HTMLDocumentNode *doc, eventid_t eid, HTMLEventObj *e
             }
         }
 
-        call_event_handlers(doc, event_obj, doc->node.event_target, &doc->basedoc.cp_container, eid,
+        call_event_handlers(doc, event_obj, doc->node.event_target.ptr, &doc->basedoc.cp_container, eid,
                 script_this ? script_this : (IDispatch*)&doc->basedoc.IHTMLDocument2_iface);
         break;
 
@@ -1295,13 +1315,6 @@ HRESULT call_fire_event(HTMLDOMNode *node, eventid_t eid)
     return S_OK;
 }
 
-static inline event_target_t *get_event_target(event_target_t **event_target_ptr)
-{
-    if(!*event_target_ptr)
-        *event_target_ptr = heap_alloc_zero(sizeof(event_target_t));
-    return *event_target_ptr;
-}
-
 static BOOL alloc_handler_vector(event_target_t *event_target, eventid_t eid, int cnt)
 {
     handler_vector_t *new_vector, *handler_vector = event_target->event_table[eid];
@@ -1362,49 +1375,77 @@ void detach_events(HTMLDocumentNode *doc)
 }
 
 
-static HRESULT remove_event_handler(event_target_t **event_target, eventid_t eid)
+static void remove_event_handler(EventTarget *event_target, eventid_t eid)
 {
-    if(*event_target && (*event_target)->event_table[eid] && (*event_target)->event_table[eid]->handler_prop) {
-        IDispatch_Release((*event_target)->event_table[eid]->handler_prop);
-        (*event_target)->event_table[eid]->handler_prop = NULL;
-    }
+    event_target_t *data;
+    VARIANT *store;
+    HRESULT hres;
 
-    return S_OK;
+    hres = dispex_get_dprop_ref(&event_target->dispex, event_info[eid].attr_name, FALSE, &store);
+    if(SUCCEEDED(hres))
+        VariantClear(store);
+
+    data = get_event_target_data(event_target, FALSE);
+    if(data && data->event_table[eid] && data->event_table[eid]->handler_prop) {
+        IDispatch_Release(data->event_table[eid]->handler_prop);
+        data->event_table[eid]->handler_prop = NULL;
+    }
 }
 
-static HRESULT set_event_handler_disp(event_target_t **event_target_ptr, HTMLDocumentNode *doc,
-        eventid_t eid, IDispatch *disp)
+static HRESULT set_event_handler_disp(EventTarget *event_target, HTMLDocumentNode *doc, eventid_t eid, IDispatch *disp)
 {
-    event_target_t *event_target;
+    event_target_t *data;
 
+    remove_event_handler(event_target, eid);
     if(!disp)
-        return remove_event_handler(event_target_ptr, eid);
+        return S_OK;
 
-    event_target = get_event_target(event_target_ptr);
+    data = get_event_target_data(event_target, TRUE);
     if(!event_target)
         return E_OUTOFMEMORY;
 
-    if(!alloc_handler_vector(event_target, eid, 0))
+    if(!alloc_handler_vector(data, eid, 0))
         return E_OUTOFMEMORY;
 
-    if(event_target->event_table[eid]->handler_prop)
-        IDispatch_Release(event_target->event_table[eid]->handler_prop);
-
-    event_target->event_table[eid]->handler_prop = disp;
+    data->event_table[eid]->handler_prop = disp;
     IDispatch_AddRef(disp);
 
-    return ensure_nsevent_handler(doc, event_target, eid);
+    return ensure_nsevent_handler(doc, data, eid);
 }
 
-HRESULT set_event_handler(event_target_t **event_target, HTMLDocumentNode *doc, eventid_t eid, VARIANT *var)
+HRESULT set_event_handler(EventTarget *event_target, HTMLDocumentNode *doc, eventid_t eid, VARIANT *var)
 {
     switch(V_VT(var)) {
     case VT_NULL:
-        return remove_event_handler(event_target, eid);
+        remove_event_handler(event_target, eid);
+        return S_OK;
 
     case VT_DISPATCH:
         return set_event_handler_disp(event_target, doc, eid, V_DISPATCH(var));
 
+    case VT_BSTR: {
+        VARIANT *v;
+        HRESULT hres;
+
+        /*
+         * Setting event handler to string is a rare case and we don't want to
+         * complicate nor increase memory of event_target_t for that. Instead,
+         * we store the value in DispatchEx, which can already handle custom
+         * properties.
+         */
+        remove_event_handler(event_target, eid);
+
+        hres = dispex_get_dprop_ref(&event_target->dispex, event_info[eid].attr_name, TRUE, &v);
+        if(FAILED(hres))
+            return hres;
+
+        V_BSTR(v) = SysAllocString(V_BSTR(var));
+        if(!V_BSTR(v))
+            return E_OUTOFMEMORY;
+        V_VT(v) = VT_BSTR;
+        return S_OK;
+    }
+
     default:
         FIXME("not handler %s\n", debugstr_variant(var));
         /* fall through */
@@ -1415,11 +1456,20 @@ HRESULT set_event_handler(event_target_t **event_target, HTMLDocumentNode *doc,
     return S_OK;
 }
 
-HRESULT get_event_handler(event_target_t **event_target, eventid_t eid, VARIANT *var)
+HRESULT get_event_handler(EventTarget *event_target, eventid_t eid, VARIANT *var)
 {
-    if(*event_target && (*event_target)->event_table[eid] && (*event_target)->event_table[eid]->handler_prop) {
+    event_target_t *data;
+    VARIANT *v;
+    HRESULT hres;
+
+    hres = dispex_get_dprop_ref(&event_target->dispex, event_info[eid].attr_name, FALSE, &v);
+    if(SUCCEEDED(hres) && V_VT(v) != VT_EMPTY)
+        return VariantCopy(var, v);
+
+    data = get_event_target_data(event_target, FALSE);
+    if(data && data->event_table[eid] && data->event_table[eid]->handler_prop) {
         V_VT(var) = VT_DISPATCH;
-        V_DISPATCH(var) = (*event_target)->event_table[eid]->handler_prop;
+        V_DISPATCH(var) = data->event_table[eid]->handler_prop;
         IDispatch_AddRef(V_DISPATCH(var));
     }else {
         V_VT(var) = VT_NULL;
@@ -1428,10 +1478,10 @@ HRESULT get_event_handler(event_target_t **event_target, eventid_t eid, VARIANT
     return S_OK;
 }
 
-HRESULT attach_event(event_target_t **event_target_ptr, HTMLDocument *doc, BSTR name,
+HRESULT attach_event(EventTarget *event_target, HTMLDocument *doc, BSTR name,
         IDispatch *disp, VARIANT_BOOL *res)
 {
-    event_target_t *event_target;
+    event_target_t *data;
     eventid_t eid;
     DWORD i = 0;
 
@@ -1442,47 +1492,49 @@ HRESULT attach_event(event_target_t **event_target_ptr, HTMLDocument *doc, BSTR
         return S_OK;
     }
 
-    event_target = get_event_target(event_target_ptr);
-    if(!event_target)
+    data = get_event_target_data(event_target, TRUE);
+    if(!data)
         return E_OUTOFMEMORY;
 
-    if(event_target->event_table[eid]) {
-        while(i < event_target->event_table[eid]->handler_cnt && event_target->event_table[eid]->handlers[i])
+    if(data->event_table[eid]) {
+        while(i < data->event_table[eid]->handler_cnt && data->event_table[eid]->handlers[i])
             i++;
-        if(i == event_target->event_table[eid]->handler_cnt && !alloc_handler_vector(event_target, eid, i+1))
+        if(i == data->event_table[eid]->handler_cnt && !alloc_handler_vector(data, eid, i+1))
             return E_OUTOFMEMORY;
-    }else if(!alloc_handler_vector(event_target, eid, i+1)) {
+    }else if(!alloc_handler_vector(data, eid, i+1)) {
         return E_OUTOFMEMORY;
     }
 
     IDispatch_AddRef(disp);
-    event_target->event_table[eid]->handlers[i] = disp;
+    data->event_table[eid]->handlers[i] = disp;
 
     *res = VARIANT_TRUE;
-    return ensure_nsevent_handler(doc->doc_node, event_target, eid);
+    return ensure_nsevent_handler(doc->doc_node, data, eid);
 }
 
-HRESULT detach_event(event_target_t *event_target, HTMLDocument *doc, BSTR name, IDispatch *disp)
+HRESULT detach_event(EventTarget *event_target, HTMLDocument *doc, BSTR name, IDispatch *disp)
 {
+    event_target_t *data;
     eventid_t eid;
     DWORD i = 0;
 
-    if(!event_target)
-        return S_OK;
-
     eid = attr_to_eid(name);
     if(eid == EVENTID_LAST) {
         WARN("Unknown event\n");
         return S_OK;
     }
 
-    if(!event_target->event_table[eid])
+    data = get_event_target_data(event_target, FALSE);
+    if(!data)
         return S_OK;
 
-    while(i < event_target->event_table[eid]->handler_cnt) {
-        if(event_target->event_table[eid]->handlers[i] == disp) {
-            IDispatch_Release(event_target->event_table[eid]->handlers[i]);
-            event_target->event_table[eid]->handlers[i] = NULL;
+    if(!data->event_table[eid])
+        return S_OK;
+
+    while(i < data->event_table[eid]->handler_cnt) {
+        if(data->event_table[eid]->handlers[i] == disp) {
+            IDispatch_Release(data->event_table[eid]->handlers[i]);
+            data->event_table[eid]->handlers[i] = NULL;
         }
         i++;
     }
@@ -1490,11 +1542,11 @@ HRESULT detach_event(event_target_t *event_target, HTMLDocument *doc, BSTR name,
     return S_OK;
 }
 
-void bind_node_event(HTMLDocumentNode *doc, event_target_t **event_target, HTMLDOMNode *node, const WCHAR *event, IDispatch *disp)
+void bind_target_event(HTMLDocumentNode *doc, EventTarget *event_target, const WCHAR *event, IDispatch *disp)
 {
     eventid_t eid;
 
-    TRACE("(%p %p %p %s %p)\n", doc, event_target, node, debugstr_w(event), disp);
+    TRACE("(%p %p %s %p)\n", doc, event_target, debugstr_w(event), disp);
 
     eid = attr_to_eid(event);
     if(eid == EVENTID_LAST) {
@@ -1505,18 +1557,18 @@ void bind_node_event(HTMLDocumentNode *doc, event_target_t **event_target, HTMLD
     set_event_handler_disp(event_target, doc, eid, disp);
 }
 
-void update_cp_events(HTMLInnerWindow *window, event_target_t **event_target_ptr, cp_static_data_t *cp)
+void update_cp_events(HTMLInnerWindow *window, EventTarget *event_target, cp_static_data_t *cp)
 {
-    event_target_t *event_target;
+    event_target_t *data;
     int i;
 
-    event_target = get_event_target(event_target_ptr);
-    if(!event_target)
+    data = get_event_target_data(event_target, FALSE);
+    if(!data)
         return; /* FIXME */
 
     for(i=0; i < EVENTID_LAST; i++) {
         if((event_info[i].flags & EVENT_DEFAULTLISTENER) && is_cp_event(cp, event_info[i].dispid))
-            ensure_nsevent_handler(window->doc, event_target, i);
+            ensure_nsevent_handler(window->doc, data, i);
     }
 }
 
@@ -1542,7 +1594,7 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem)
             if(disp) {
                 hres = get_node(doc, (nsIDOMNode*)nselem, TRUE, &node);
                 if(SUCCEEDED(hres)) {
-                    set_event_handler_disp(get_node_event_target(node), node->doc, i, disp);
+                    set_event_handler_disp(&node->event_target, node->doc, i, disp);
                     node_release(node);
                 }
                 IDispatch_Release(disp);
