@@ -724,7 +724,7 @@ static void test_query_object(void)
     todo_wine ok( len > sizeof(OBJECT_TYPE_INFORMATION), "unexpected len %u\n", len );
     str = (UNICODE_STRING *)buffer;
     todo_wine ok( len >= sizeof(OBJECT_TYPE_INFORMATION) + str->Length + sizeof(WCHAR), "unexpected len %u\n", len );
-    todo_wine ok( str->Buffer && !memcmp( str->Buffer, type_event, sizeof(type_file) ),
+    todo_wine ok( str->Buffer && !memcmp( str->Buffer, type_event, sizeof(type_event) ),
                   "wrong/bad type name %s (%p)\n", wine_dbgstr_w(str->Buffer), str->Buffer );
 
     len -= sizeof(WCHAR);
@@ -835,14 +835,14 @@ static void test_event(void)
     status = pNtOpenEvent(&Event2, GENERIC_ALL, &attr);
     ok( status == STATUS_SUCCESS, "NtOpenEvent failed %08x\n", status );
 
-    status = pNtClose(Event);
+    pNtClose(Event);
 
     status = pNtQueryEvent(Event2, EventBasicInformation, &info, sizeof(info), NULL);
     ok( status == STATUS_SUCCESS, "NtQueryEvent failed %08x\n", status );
     ok( info.EventType == 1 && info.EventState == 0,
         "NtQueryEvent failed, expected 1 0, got %d %d\n", info.EventType, info.EventState );
 
-    status = pNtClose(Event2);
+    pNtClose(Event2);
 }
 
 static const WCHAR keyed_nameW[] = {'\\','B','a','s','e','N','a','m','e','d','O','b','j','e','c','t','s',
