@@ -78,6 +78,7 @@ static const WORD  MAX_BIG_BLOCK_SIZE_BITS   = 0x000c;
 static const WORD  DEF_SMALL_BLOCK_SIZE_BITS = 0x0006;
 static const WORD  DEF_BIG_BLOCK_SIZE        = 0x0200;
 static const WORD  DEF_SMALL_BLOCK_SIZE      = 0x0040;
+static const ULONG BLOCK_FIRST_SPECIAL       = 0xFFFFFFFB;
 static const ULONG BLOCK_EXTBBDEPOT          = 0xFFFFFFFC;
 static const ULONG BLOCK_SPECIAL             = 0xFFFFFFFD;
 static const ULONG BLOCK_END_OF_CHAIN        = 0xFFFFFFFE;
@@ -158,8 +159,6 @@ struct DirEntry
 
 HRESULT FileLockBytesImpl_Construct(HANDLE hFile, DWORD openFlags, LPCWSTR pwcsName, ILockBytes **pLockBytes) DECLSPEC_HIDDEN;
 
-HRESULT FileLockBytesImpl_LockRegionSync(ILockBytes* iface, ULARGE_INTEGER libOffset, ULARGE_INTEGER cb) DECLSPEC_HIDDEN;
-
 /*************************************************************************
  * Ole Convert support
  */
@@ -364,9 +363,9 @@ void StorageBaseImpl_RemoveStream(StorageBaseImpl * stg, StgStreamImpl * strm) D
 #define BLOCKCHAIN_CACHE_SIZE 4
 
 /****************************************************************************
- * Storage32Impl definitions.
+ * StorageImpl definitions.
  *
- * This implementation of the IStorage32 interface represents a root
+ * This implementation of the IStorage interface represents a root
  * storage. Basically, a document file.
  */
 struct StorageImpl
@@ -417,39 +416,6 @@ struct StorageImpl
   ULONG locked_bytes[8];
 };
 
-HRESULT StorageImpl_ReadRawDirEntry(
-            StorageImpl *This,
-            ULONG index,
-            BYTE *buffer) DECLSPEC_HIDDEN;
-
-void UpdateRawDirEntry(
-    BYTE *buffer,
-    const DirEntry *newData) DECLSPEC_HIDDEN;
-
-HRESULT StorageImpl_WriteRawDirEntry(
-            StorageImpl *This,
-            ULONG index,
-            const BYTE *buffer) DECLSPEC_HIDDEN;
-
-HRESULT StorageImpl_ReadDirEntry(
-            StorageImpl*    This,
-            DirRef          index,
-            DirEntry*       buffer) DECLSPEC_HIDDEN;
-
-HRESULT StorageImpl_WriteDirEntry(
-            StorageImpl*        This,
-            DirRef              index,
-            const DirEntry*     buffer) DECLSPEC_HIDDEN;
-
-BlockChainStream* Storage32Impl_SmallBlocksToBigBlocks(
-                      StorageImpl* This,
-                      SmallBlockChainStream** ppsbChain) DECLSPEC_HIDDEN;
-
-SmallBlockChainStream* Storage32Impl_BigBlocksToSmallBlocks(
-                      StorageImpl* This,
-                      BlockChainStream** ppbbChain,
-                      ULARGE_INTEGER newSize) DECLSPEC_HIDDEN;
-
 /****************************************************************************
  * StgStreamImpl definitions.
  *
@@ -504,7 +470,10 @@ StgStreamImpl* StgStreamImpl_Construct(
 /* Range lock constants.
  *
  * The storage format reserves the region from 0x7fffff00-0x7fffffff for
- * locking and synchronization. Unfortunately, the spec doesn't say which bytes
+ * locking and synchronization. Because it reserves the entire block containing
+ * that range, and the minimum block size is 512 bytes, 0x7ffffe00-0x7ffffeff
+ * also cannot be used for any other purpose.
+ * Unfortunately, the spec doesn't say which bytes
  * within that range are used, and for what. These are guesses based on testing.
  * In particular, ends of ranges may be wrong.
 
@@ -523,7 +492,7 @@ StgStreamImpl* StgStreamImpl_Construct(
  0xe2 through 0xff: Unknown. Causes read-only exclusive opens to fail.
 */
 
-#define RANGELOCK_UNK1_FIRST            0x7fffff00
+#define RANGELOCK_UNK1_FIRST            0x7ffffe00
 #define RANGELOCK_UNK1_LAST             0x7fffff57
 #define RANGELOCK_PRIORITY1_FIRST       0x7fffff58
 #define RANGELOCK_PRIORITY1_LAST        0x7fffff6b
@@ -586,116 +555,5 @@ void StorageUtl_WriteGUID(BYTE* buffer, ULONG offset, const GUID* value) DECLSPE
 void StorageUtl_CopyDirEntryToSTATSTG(StorageBaseImpl *storage,STATSTG* destination,
  const DirEntry* source, int statFlags) DECLSPEC_HIDDEN;
 
-/****************************************************************************
- * BlockChainStream definitions.
- *
- * The BlockChainStream class is a utility class that is used to create an
- * abstraction of the big block chains in the storage file.
- */
-struct BlockChainRun
-{
-  /* This represents a range of blocks that happen reside in consecutive sectors. */
-  ULONG firstSector;
-  ULONG firstOffset;
-  ULONG lastOffset;
-};
-
-typedef struct BlockChainBlock
-{
-  ULONG index;
-  ULONG sector;
-  BOOL  read;
-  BOOL  dirty;
-  BYTE data[MAX_BIG_BLOCK_SIZE];
-} BlockChainBlock;
-
-struct BlockChainStream
-{
-  StorageImpl* parentStorage;
-  ULONG*       headOfStreamPlaceHolder;
-  DirRef       ownerDirEntry;
-  struct BlockChainRun* indexCache;
-  ULONG        indexCacheLen;
-  ULONG        indexCacheSize;
-  BlockChainBlock cachedBlocks[2];
-  ULONG        blockToEvict;
-  ULONG        tailIndex;
-  ULONG        numBlocks;
-};
-
-/*
- * Methods for the BlockChainStream class.
- */
-BlockChainStream* BlockChainStream_Construct(
-		StorageImpl* parentStorage,
-		ULONG*         headOfStreamPlaceHolder,
-		DirRef         dirEntry) DECLSPEC_HIDDEN;
-
-void BlockChainStream_Destroy(
-		BlockChainStream* This) DECLSPEC_HIDDEN;
-
-HRESULT BlockChainStream_ReadAt(
-		BlockChainStream* This,
-		ULARGE_INTEGER offset,
-		ULONG          size,
-		void*          buffer,
-		ULONG*         bytesRead) DECLSPEC_HIDDEN;
-
-HRESULT BlockChainStream_WriteAt(
-		BlockChainStream* This,
-		ULARGE_INTEGER offset,
-		ULONG          size,
-		const void*    buffer,
-		ULONG*         bytesWritten) DECLSPEC_HIDDEN;
-
-BOOL BlockChainStream_SetSize(
-		BlockChainStream* This,
-		ULARGE_INTEGER    newSize) DECLSPEC_HIDDEN;
-
-HRESULT BlockChainStream_Flush(
-                BlockChainStream* This) DECLSPEC_HIDDEN;
-
-/****************************************************************************
- * SmallBlockChainStream definitions.
- *
- * The SmallBlockChainStream class is a utility class that is used to create an
- * abstraction of the small block chains in the storage file.
- */
-struct SmallBlockChainStream
-{
-  StorageImpl* parentStorage;
-  DirRef         ownerDirEntry;
-  ULONG*         headOfStreamPlaceHolder;
-};
-
-/*
- * Methods of the SmallBlockChainStream class.
- */
-SmallBlockChainStream* SmallBlockChainStream_Construct(
-           StorageImpl*   parentStorage,
-           ULONG*         headOfStreamPlaceHolder,
-           DirRef         dirEntry) DECLSPEC_HIDDEN;
-
-void SmallBlockChainStream_Destroy(
-	       SmallBlockChainStream* This) DECLSPEC_HIDDEN;
-
-HRESULT SmallBlockChainStream_ReadAt(
-	       SmallBlockChainStream* This,
-	       ULARGE_INTEGER offset,
-	       ULONG          size,
-	       void*          buffer,
-	       ULONG*         bytesRead) DECLSPEC_HIDDEN;
-
-HRESULT SmallBlockChainStream_WriteAt(
-	       SmallBlockChainStream* This,
-	       ULARGE_INTEGER offset,
-	       ULONG          size,
-	       const void*    buffer,
-	       ULONG*         bytesWritten) DECLSPEC_HIDDEN;
-
-BOOL SmallBlockChainStream_SetSize(
-	       SmallBlockChainStream* This,
-	       ULARGE_INTEGER          newSize) DECLSPEC_HIDDEN;
-
 
 #endif /* __STORAGE32_H__ */
