@@ -22,7 +22,6 @@
 #include <stdarg.h>
 
 #define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #define COBJMACROS
 
 #include "windef.h"
@@ -52,8 +51,10 @@ typedef struct IOleClientSiteImpl IOleClientSiteImpl;
 typedef struct ITextRangeImpl ITextRangeImpl;
 
 typedef struct IRichEditOleImpl {
+    IUnknown IUnknown_inner;
     IRichEditOle IRichEditOle_iface;
     ITextDocument ITextDocument_iface;
+    IUnknown *outer_unk;
     LONG ref;
 
     ME_TextEditor *editor;
@@ -80,6 +81,8 @@ struct ITextSelectionImpl {
 
 struct IOleClientSiteImpl {
     IOleClientSite IOleClientSite_iface;
+    IOleWindow IOleWindow_iface;
+    IOleInPlaceSite IOleInPlaceSite_iface;
     LONG ref;
 
     IRichEditOleImpl *reOle;
@@ -95,44 +98,47 @@ static inline IRichEditOleImpl *impl_from_ITextDocument(ITextDocument *iface)
     return CONTAINING_RECORD(iface, IRichEditOleImpl, ITextDocument_iface);
 }
 
-static HRESULT WINAPI
-IRichEditOle_fnQueryInterface(IRichEditOle *me, REFIID riid, LPVOID *ppvObj)
+static inline IRichEditOleImpl *impl_from_IUnknown(IUnknown *iface)
 {
-    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
+    return CONTAINING_RECORD(iface, IRichEditOleImpl, IUnknown_inner);
+}
 
-    TRACE("%p %s\n", This, debugstr_guid(riid) );
+static HRESULT WINAPI IRichEditOleImpl_inner_fnQueryInterface(IUnknown *iface, REFIID riid, LPVOID *ppvObj)
+{
+    IRichEditOleImpl *This = impl_from_IUnknown(iface);
+
+    TRACE("%p %s\n", This, debugstr_guid(riid));
 
     *ppvObj = NULL;
-    if (IsEqualGUID(riid, &IID_IUnknown) ||
-        IsEqualGUID(riid, &IID_IRichEditOle))
+    if (IsEqualGUID(riid, &IID_IUnknown))
+        *ppvObj = &This->IUnknown_inner;
+    else if (IsEqualGUID(riid, &IID_IRichEditOle))
         *ppvObj = &This->IRichEditOle_iface;
     else if (IsEqualGUID(riid, &IID_ITextDocument))
         *ppvObj = &This->ITextDocument_iface;
     if (*ppvObj)
     {
-        IRichEditOle_AddRef(me);
+        IUnknown_AddRef((IUnknown *)*ppvObj);
         return S_OK;
     }
-    FIXME("%p: unhandled interface %s\n", This, debugstr_guid(riid) );
+    FIXME("%p: unhandled interface %s\n", This, debugstr_guid(riid));
  
     return E_NOINTERFACE;   
 }
 
-static ULONG WINAPI
-IRichEditOle_fnAddRef(IRichEditOle *me)
+static ULONG WINAPI IRichEditOleImpl_inner_fnAddRef(IUnknown *iface)
 {
-    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
-    ULONG ref = InterlockedIncrement( &This->ref );
+    IRichEditOleImpl *This = impl_from_IUnknown(iface);
+    ULONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("%p ref = %u\n", This, ref);
 
     return ref;
 }
 
-static ULONG WINAPI
-IRichEditOle_fnRelease(IRichEditOle *me)
+static ULONG WINAPI IRichEditOleImpl_inner_fnRelease(IUnknown *iface)
 {
-    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
+    IRichEditOleImpl *This = impl_from_IUnknown(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
     TRACE ("%p ref=%u\n", This, ref);
@@ -140,7 +146,8 @@ IRichEditOle_fnRelease(IRichEditOle *me)
     if (!ref)
     {
         ITextRangeImpl *txtRge;
-        TRACE ("Destroying %p\n", This);
+
+        TRACE("Destroying %p\n", This);
         This->txtSel->reOle = NULL;
         This->editor->reOle = NULL;
         ITextSelection_Release(&This->txtSel->ITextSelection_iface);
@@ -152,6 +159,34 @@ IRichEditOle_fnRelease(IRichEditOle *me)
     return ref;
 }
 
+static const IUnknownVtbl reo_unk_vtbl =
+{
+    IRichEditOleImpl_inner_fnQueryInterface,
+    IRichEditOleImpl_inner_fnAddRef,
+    IRichEditOleImpl_inner_fnRelease
+};
+
+static HRESULT WINAPI
+IRichEditOle_fnQueryInterface(IRichEditOle *me, REFIID riid, LPVOID *ppvObj)
+{
+    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
+    return IUnknown_QueryInterface(This->outer_unk, riid, ppvObj);
+}
+
+static ULONG WINAPI
+IRichEditOle_fnAddRef(IRichEditOle *me)
+{
+    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
+    return IUnknown_AddRef(This->outer_unk);
+}
+
+static ULONG WINAPI
+IRichEditOle_fnRelease(IRichEditOle *me)
+{
+    IRichEditOleImpl *This = impl_from_IRichEditOle(me);
+    return IUnknown_Release(This->outer_unk);
+}
+
 static HRESULT WINAPI
 IRichEditOle_fnActivateAs(IRichEditOle *me, REFCLSID rclsid, REFCLSID rclsidAs)
 {
@@ -185,12 +220,17 @@ static inline IOleClientSiteImpl *impl_from_IOleClientSite(IOleClientSite *iface
 static HRESULT WINAPI
 IOleClientSite_fnQueryInterface(IOleClientSite *me, REFIID riid, LPVOID *ppvObj)
 {
+    IOleClientSiteImpl *This = impl_from_IOleClientSite(me);
     TRACE("%p %s\n", me, debugstr_guid(riid) );
 
     *ppvObj = NULL;
     if (IsEqualGUID(riid, &IID_IUnknown) ||
         IsEqualGUID(riid, &IID_IOleClientSite))
         *ppvObj = me;
+    else if (IsEqualGUID(riid, &IID_IOleWindow))
+        *ppvObj = &This->IOleWindow_iface;
+    else if (IsEqualGUID(riid, &IID_IOleInPlaceSite))
+        *ppvObj = &This->IOleInPlaceSite_iface;
     if (*ppvObj)
     {
         IOleClientSite_AddRef(me);
@@ -291,6 +331,184 @@ static const IOleClientSiteVtbl ocst = {
     IOleClientSite_fnRequestNewObjectLayout
 };
 
+/* IOleWindow interface */
+static inline IOleClientSiteImpl *impl_from_IOleWindow(IOleWindow *iface)
+{
+    return CONTAINING_RECORD(iface, IOleClientSiteImpl, IOleWindow_iface);
+}
+
+static HRESULT WINAPI IOleWindow_fnQueryInterface(IOleWindow *iface, REFIID riid, void **ppvObj)
+{
+    IOleClientSiteImpl *This = impl_from_IOleWindow(iface);
+    return IOleClientSite_QueryInterface(&This->IOleClientSite_iface, riid, ppvObj);
+}
+
+static ULONG WINAPI IOleWindow_fnAddRef(IOleWindow *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleWindow(iface);
+    return IOleClientSite_AddRef(&This->IOleClientSite_iface);
+}
+
+static ULONG WINAPI IOleWindow_fnRelease(IOleWindow *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleWindow(iface);
+    return IOleClientSite_Release(&This->IOleClientSite_iface);
+}
+
+static HRESULT WINAPI IOleWindow_fnContextSensitiveHelp(IOleWindow *iface, BOOL fEnterMode)
+{
+    IOleClientSiteImpl *This = impl_from_IOleWindow(iface);
+    FIXME("not implemented: (%p)->(%d)\n", This, fEnterMode);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI IOleWindow_fnGetWindow(IOleWindow *iface, HWND *phwnd)
+{
+    IOleClientSiteImpl *This = impl_from_IOleWindow(iface);
+    TRACE("(%p)->(%p)\n", This, phwnd);
+
+    if (!phwnd)
+        return E_INVALIDARG;
+
+    *phwnd = This->reOle->editor->hWnd;
+    return S_OK;
+}
+
+static const IOleWindowVtbl olewinvt = {
+    IOleWindow_fnQueryInterface,
+    IOleWindow_fnAddRef,
+    IOleWindow_fnRelease,
+    IOleWindow_fnGetWindow,
+    IOleWindow_fnContextSensitiveHelp
+};
+
+/* IOleInPlaceSite interface */
+static inline IOleClientSiteImpl *impl_from_IOleInPlaceSite(IOleInPlaceSite *iface)
+{
+    return CONTAINING_RECORD(iface, IOleClientSiteImpl, IOleInPlaceSite_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnQueryInterface(IOleInPlaceSite *iface, REFIID riid, void **ppvObj)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    return IOleClientSite_QueryInterface(&This->IOleClientSite_iface, riid, ppvObj);
+}
+
+static ULONG STDMETHODCALLTYPE IOleInPlaceSite_fnAddRef(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    return IOleClientSite_AddRef(&This->IOleClientSite_iface);
+}
+
+static ULONG STDMETHODCALLTYPE IOleInPlaceSite_fnRelease(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    return IOleClientSite_Release(&This->IOleClientSite_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnGetWindow(IOleInPlaceSite *iface, HWND *phwnd)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    return IOleWindow_GetWindow(&This->IOleWindow_iface, phwnd);
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnContextSensitiveHelp(IOleInPlaceSite *iface, BOOL fEnterMode)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    return IOleWindow_ContextSensitiveHelp(&This->IOleWindow_iface, fEnterMode);
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnCanInPlaceActivate(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnOnInPlaceActivate(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnOnUIActivate(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnGetWindowContext(IOleInPlaceSite *iface, IOleInPlaceFrame **ppFrame,
+                                                                    IOleInPlaceUIWindow **ppDoc, LPRECT lprcPosRect,
+                                                                    LPRECT lprcClipRect, LPOLEINPLACEFRAMEINFO lpFrameInfo)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)->(%p %p %p %p %p)\n", This, ppFrame, ppDoc, lprcPosRect, lprcClipRect, lpFrameInfo);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnScroll(IOleInPlaceSite *iface, SIZE scrollExtent)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnOnUIDeactivate(IOleInPlaceSite *iface, BOOL fUndoable)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)->(%d)\n", This, fUndoable);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnOnInPlaceDeactivate(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnDiscardUndoState(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnDeactivateAndUndo(IOleInPlaceSite *iface)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE IOleInPlaceSite_fnOnPosRectChange(IOleInPlaceSite *iface, LPCRECT lprcPosRect)
+{
+    IOleClientSiteImpl *This = impl_from_IOleInPlaceSite(iface);
+    FIXME("not implemented: (%p)->(%p)\n", This, lprcPosRect);
+    return E_NOTIMPL;
+}
+
+static const IOleInPlaceSiteVtbl olestvt =
+{
+    IOleInPlaceSite_fnQueryInterface,
+    IOleInPlaceSite_fnAddRef,
+    IOleInPlaceSite_fnRelease,
+    IOleInPlaceSite_fnGetWindow,
+    IOleInPlaceSite_fnContextSensitiveHelp,
+    IOleInPlaceSite_fnCanInPlaceActivate,
+    IOleInPlaceSite_fnOnInPlaceActivate,
+    IOleInPlaceSite_fnOnUIActivate,
+    IOleInPlaceSite_fnGetWindowContext,
+    IOleInPlaceSite_fnScroll,
+    IOleInPlaceSite_fnOnUIDeactivate,
+    IOleInPlaceSite_fnOnInPlaceDeactivate,
+    IOleInPlaceSite_fnDiscardUndoState,
+    IOleInPlaceSite_fnDeactivateAndUndo,
+    IOleInPlaceSite_fnOnPosRectChange
+};
+
 static IOleClientSiteImpl *
 CreateOleClientSite(IRichEditOleImpl *reOle)
 {
@@ -299,6 +517,8 @@ CreateOleClientSite(IRichEditOleImpl *reOle)
         return NULL;
 
     clientSite->IOleClientSite_iface.lpVtbl = &ocst;
+    clientSite->IOleWindow_iface.lpVtbl = &olewinvt;
+    clientSite->IOleInPlaceSite_iface.lpVtbl = &olestvt;
     clientSite->ref = 1;
     clientSite->reOle = reOle;
     return clientSite;
@@ -2305,7 +2525,7 @@ CreateTextSelection(IRichEditOleImpl *reOle)
     return txtSel;
 }
 
-LRESULT CreateIRichEditOle(ME_TextEditor *editor, LPVOID *ppObj)
+LRESULT CreateIRichEditOle(IUnknown *outer_unk, ME_TextEditor *editor, LPVOID *ppvObj)
 {
     IRichEditOleImpl *reo;
 
@@ -2313,6 +2533,7 @@ LRESULT CreateIRichEditOle(ME_TextEditor *editor, LPVOID *ppObj)
     if (!reo)
         return 0;
 
+    reo->IUnknown_inner.lpVtbl = &reo_unk_vtbl;
     reo->IRichEditOle_iface.lpVtbl = &revt;
     reo->ITextDocument_iface.lpVtbl = &tdvt;
     reo->ref = 1;
@@ -2331,8 +2552,12 @@ LRESULT CreateIRichEditOle(ME_TextEditor *editor, LPVOID *ppObj)
         return 0;
     }
     TRACE("Created %p\n",reo);
-    *ppObj = reo;
     list_init(&reo->rangelist);
+    if (outer_unk)
+        reo->outer_unk = outer_unk;
+    else
+        reo->outer_unk = &reo->IUnknown_inner;
+    *ppvObj = &reo->IRichEditOle_iface;
 
     return 1;
 }
@@ -2540,3 +2765,9 @@ void ME_CopyReObject(REOBJECT* dst, const REOBJECT* src)
     if (dst->pstg)      IStorage_AddRef(dst->pstg);
     if (dst->polesite)  IOleClientSite_AddRef(dst->polesite);
 }
+
+void ME_GetITextDocumentInterface(IRichEditOle *iface, LPVOID *ppvObj)
+{
+    IRichEditOleImpl *This = impl_from_IRichEditOle(iface);
+    *ppvObj = &This->ITextDocument_iface;
+}
