@@ -422,20 +422,15 @@ static BOOL application_octet_stream_filter(const BYTE *b, DWORD size)
     return TRUE;
 }
 
-static HRESULT find_mime_from_url(const WCHAR *url, WCHAR **ret)
+HRESULT find_mime_from_ext(const WCHAR *ext, WCHAR **ret)
 {
-    const WCHAR *ptr;
     DWORD res, size;
     WCHAR mime[64];
     HKEY hkey;
 
     static const WCHAR content_typeW[] = {'C','o','n','t','e','n','t',' ','T','y','p','e','\0'};
 
-    ptr = strrchrW(url, '.');
-    if(!ptr)
-        return E_FAIL;
-
-    res = RegOpenKeyW(HKEY_CLASSES_ROOT, ptr, &hkey);
+    res = RegOpenKeyW(HKEY_CLASSES_ROOT, ext, &hkey);
     if(res != ERROR_SUCCESS)
         return HRESULT_FROM_WIN32(res);
 
@@ -452,6 +447,41 @@ static HRESULT find_mime_from_url(const WCHAR *url, WCHAR **ret)
     return S_OK;
 }
 
+static HRESULT find_mime_from_url(const WCHAR *url, WCHAR **ret)
+{
+    const WCHAR *ptr, *end_ptr;
+    WCHAR *ext = NULL;
+    HRESULT hres;
+
+    for(end_ptr = url; *end_ptr; end_ptr++) {
+        if(*end_ptr == '?' || *end_ptr == '#')
+            break;
+    }
+
+    for(ptr = end_ptr; ptr >= url; ptr--) {
+        if(*ptr == '.')
+            break;
+    }
+
+    if(ptr < url)
+        return E_FAIL;
+
+    if(*end_ptr) {
+        unsigned len = end_ptr-ptr;
+
+        ext = heap_alloc((len+1)*sizeof(WCHAR));
+        if(!ext)
+            return E_OUTOFMEMORY;
+
+        memcpy(ext, ptr, len*sizeof(WCHAR));
+        ext[len] = 0;
+    }
+
+    hres = find_mime_from_ext(ext ? ext : ptr, ret);
+    heap_free(ext);
+    return hres;
+}
+
 static const WCHAR text_htmlW[] = {'t','e','x','t','/','h','t','m','l',0};
 static const WCHAR text_richtextW[] = {'t','e','x','t','/','r','i','c','h','t','e','x','t',0};
 static const WCHAR text_xmlW[] = {'t','e','x','t','/','x','m','l',0};
