@@ -458,6 +458,7 @@ ULONG CDECL wined3d_stateblock_incref(struct wined3d_stateblock *stateblock)
 
 void state_unbind_resources(struct wined3d_state *state)
 {
+    struct wined3d_shader_resource_view *srv;
     struct wined3d_vertex_declaration *decl;
     struct wined3d_sampler *sampler;
     struct wined3d_texture *texture;
@@ -529,6 +530,15 @@ void state_unbind_resources(struct wined3d_state *state)
                 wined3d_sampler_decref(sampler);
             }
         }
+
+        for (j = 0; j < MAX_SHADER_RESOURCE_VIEWS; ++j)
+        {
+            if ((srv = state->shader_resource_view[i][j]))
+            {
+                state->shader_resource_view[i][j] = NULL;
+                wined3d_shader_resource_view_decref(srv);
+            }
+        }
     }
 }
 
@@ -1111,16 +1121,11 @@ static void state_init_default(struct wined3d_state *state, const struct wined3d
         DWORD d;
     } tmpfloat;
     unsigned int i;
-    static const struct wined3d_matrix identity =
-    {{{
-        1.0f, 0.0f, 0.0f, 0.0f,
-        0.0f, 1.0f, 0.0f, 0.0f,
-        0.0f, 0.0f, 1.0f, 0.0f,
-        0.0f, 0.0f, 0.0f, 1.0f,
-    }}};
+    struct wined3d_matrix identity;
 
     TRACE("state %p, gl_info %p.\n", state, gl_info);
 
+    get_identity_matrix(&identity);
     state->gl_primitive_type = ~0u;
 
     /* Set some of the defaults for lights, transforms etc */
